<?php

namespace AustraliaPost\Extensions\Business\Endpoints;

class Get_Shipments_By_Date_Endpoint extends Abstract_Endpoint {


	/**
	 * @param array $data
	 *
	 * @return array|void
	 */
	public function callback( $data )
	{
		$this->validate($data);

		$from = $data['date'][0];
		$to = $data['date'][1];

		$orders = $this->get_order_ids($from, $to);

		if (count($orders) === 0) {
			return [];
		}

		return $this->get_shipments($orders);
	}

	/**
	 * @return string
	 */
	public function action()
	{
		return 'get_shipments_by_date';
	}

	private function validate( array $data )
	{
		if (!isset($data['date'])) {
			$this->abort(['date is not provided']);
		}

		if (count($data['date']) !== 2) {
			$this->abort(['date range is not provided']);
		}
	}

	/**
	 * @param string $from
	 * @param string $to
	 *
	 * @return array
	 */
	private function get_order_ids( $from, $to )
	{
		global $wpdb;
		$key  = Create_Shipments_Endpoint::LABELS_SHIPMENTS_DATE_META_KEY;
		return $wpdb->get_col(
			$wpdb->prepare("SELECT post_id FROM {$wpdb->prefix}postmeta WHERE meta_key = %s AND DATE(meta_value) BETWEEN %s AND %s", [$key, $from, $to])
		);
	}

	private function get_shipments( $orders )
	{
		global $wpdb;
		$orders = implode(',', array_map('intval', $orders));
		$key = Create_Shipments_Endpoint::LABELS_SHIPMENTS_META_KEY;
		$results = $wpdb->get_col(
			$wpdb->prepare("SELECT meta_value FROM {$wpdb->prefix}postmeta WHERE meta_key = %s AND post_id IN (". $orders .")", [$key])
		);

		$ordered_shipments = get_option(Create_Order_From_Shipments_Endpoint::OPTION_KEY_ORDERED_SHIPMENTS, []);

		$shipments = [];
		foreach ($results as $result) {
			$result = json_decode($result, true);
			foreach ($result as $shipment) {
				if ( !in_array($shipment['shipment_id'], $ordered_shipments)) {
					if (isset($shipment['shipment_reference'])) {
						$shipment['wc_order_id'] = $this->get_order_id_from_shipment_reference($shipment['shipment_reference']);
					}
					$shipments[] = $shipment;
				}
			}
		}

		return $shipments;
	}

}
